#define MAIN_FILE
#include <inttypes.h>
#include "mex.h"
#include <simstruc.h> // needed to forward printf output to matlab

/* Returns the number of elements inside a community described by N, H, and
 * gamma.
 *
 * This function is intended to be called from Matlab.
 */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
    if (nrhs != 3)
        mexErrMsgTxt("Wrong number of input arguments. Exactly three arguments expected.");
    if (nlhs != 1)
        mexErrMsgTxt("Wrong number of output arguments. Exactly one argument expected.");

    const int64_t N = (int64_t) mxGetScalar(prhs[0]);
    const int64_t H = (int64_t) mxGetScalar(prhs[1]);
    const int64_t gamma = (int64_t) mxGetScalar(prhs[2]);
	
    int64_t area = (H+1)*N-(H+1)-((H+1)*(H+1)-(H+1))/2;

    for (int64_t i = H+1; i < gamma; i++) {
        // derived from maxIndex = floor(theta/(i+p)-p) for integer division
        const int64_t gamma2NH = gamma*gamma-(N-1)*H;
        const int64_t NH2gamma = N-1+H - 2*gamma;
        const int64_t iNHgamma = i*NH2gamma+gamma2NH;
        const int64_t maxIndex = ((gamma-H)*(gamma-H)*(gamma-(N-1))*(gamma-(N-1))-gamma2NH*iNHgamma) / (NH2gamma*iNHgamma);
        area = area + maxIndex - i;
    }

    /* Prepare output. */
    plhs[0] = mxCreateDoubleScalar( (double) area);
    return;
}