function [ R, succ ] = swap_rand( G, K )
%SWAP_RAND randomizes a graph using swap randomization
%   R = SWAP_RAND(G, k) takes (symmetric) adjacency matrix G and number of
%   steps k and returns a random graph R with the same node degrees as G.
%
%   [R, succ] = SWAP_RAND(G, k) will also return the number of successfull
%   swaps.

if ~issymmetric(G)
    error('Adjacency matrix must be symmetric');
end

n = size(G, 1);

R = G;
succ = 0;


for rep=1:K
    nodes = randsample(n, 2);
    i = nodes(1);
    k = nodes(2);
    gammaI = R(:,i);
    gammaK = R(:,k);
    gammaI(i) = 0; % no self-edges
    gammaK(k) = 0;
    gammaI(k) = 0; % Don't allow i or k as l or j
    gammaK(i) = 0;
    if nnz(gammaI) == 0 
        continue; % these nodes have no neighbours
    end
    neighI = find(gammaI);
    j = neighI(randsample(length(neighI), 1)); % edge (i, j)
    gammaK(j) = 0;
    if nnz(gammaK) == 0
        continue
    end
    neighK = find(gammaK);
    l = neighK(randsample(length(neighK), 1)); % edge (k, l)

    if R(i, l) == 0 && R(k, j) == 0
        % Swap edges
        R(i, j) = 0;
        R(j, i) = 0;
        R(k, l) = 0;
        R(l, k) = 0;
        R(i, l) = 1;
        R(l, i) = 1;
        R(k, j) = 1;
        R(j, k) = 1;
        succ = succ + 1;
    end
end

